package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.entity.BasicCourse;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.entity.SysUser;
import com.qmth.distributed.print.business.mapper.BasicCourseMapper;
import com.qmth.distributed.print.business.service.BasicCourseService;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 课程表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicCourseServiceImpl extends ServiceImpl<BasicCourseMapper, BasicCourse> implements BasicCourseService {

    @Autowired
    private ExamTaskService examTaskService;

    @Override
    public IPage<BasicCourse> list(String code, String name, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Page<BasicCourse> page = new Page<>(pageNumber, pageSize);
        IPage<BasicCourse> courseIPage = this.baseMapper.listPage(page, schoolId, code, name);
        return courseIPage;
    }

    @Override
    public List<BasicCourse> list(String param) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(param)) {
            queryWrapper.lambda().eq(BasicCourse::getSchoolId, schoolId)
                    .and(i -> i.like(BasicCourse::getCode, param).or().like(BasicCourse::getName, param));
        }
        return this.list(queryWrapper);
    }

    @Override
    public boolean saveCourse(BasicCourse course) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        course.setSchoolId(schoolId);
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicCourse::getSchoolId, course.getSchoolId()).eq(BasicCourse::getCode, course.getCode());
        BasicCourse basicCourse = this.getOne(queryWrapper);

        // 新增
        if (Objects.isNull(course.getId())) {
            if (basicCourse != null) {
                throw ExceptionResultEnum.ERROR.exception("课程已存在");
            }
            course.setId(SystemConstant.getDbUuid());
            course.setCreateId(sysUser.getId());
            course.setCreateTime(System.currentTimeMillis());
        }
        // 修改
        else {
            if (basicCourse != null && basicCourse.getId().longValue() != course.getId().longValue()) {
                throw ExceptionResultEnum.ERROR.exception("课程已存在");
            }

            // 旧code是否已有命题任务
            BasicCourse oldCourse = this.getById(course.getId());
            List<ExamTask> taskList = examTaskService.listByCourseCode(oldCourse.getSchoolId(), oldCourse.getCode());
            if (taskList != null && taskList.size() > 0) {
                throw ExceptionResultEnum.ERROR.exception("课程绑定命题任务，不能修改");
            }
            course.setUpdateId(sysUser.getId());
            course.setUpdateTime(System.currentTimeMillis());
        }
        return this.saveOrUpdate(course);
    }

    @Override
    public boolean remove(Long id) {
        // code是否已有命题任务
        BasicCourse course = this.baseMapper.selectById(id);
        List<ExamTask> taskList = examTaskService.listByCourseCode(course.getSchoolId(), course.getCode());
        if (taskList != null && taskList.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("课程绑定命题任务，不能删除");
        }
        return this.removeById(id);
    }

    @Override
    public List<BasicCourse> listCoursesByUserId(Long userId) {
        return this.baseMapper.listCoursesByUserId(userId);
    }
}
